package com.example.webflux;

import org.springframework.boot.ApplicationRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.r2dbc.core.DatabaseClient;
import org.springframework.web.reactive.function.server.RouterFunction;
import org.springframework.web.reactive.function.server.ServerResponse;

import static org.springframework.web.reactive.function.server.RouterFunctions.route;
import static org.springframework.web.reactive.function.server.ServerResponse.ok;

@SpringBootApplication(proxyBeanMethods = false)
public class WebfluxApplication {

	@Bean
	RouterFunction<ServerResponse> routes(ReservationRepository reservationRepository) {
		return route().GET("/reservations", r -> ok().body(reservationRepository.findAll(), Reservation.class)).build();
	}

	@Bean
	ApplicationRunner runner(DatabaseClient dbc, ReservationRepository reservationRepository) {
		return args -> {

			dbc.sql("create table reservation\n" + "(\n" + "    id   serial primary key,\n"
					+ "    name varchar(255) not null\n" + ")").fetch().rowsUpdated()
					.then(reservationRepository.save(new Reservation(null, "Andy")))
					.then(reservationRepository.save(new Reservation(null, "Sebastien")))
					.then(reservationRepository.findAll().doOnNext(System.out::println).then()).subscribe();
		};
	}

	public static void main(String[] args) {
		SpringApplication.run(WebfluxApplication.class, args);
	}
}
